%function C=cluster(X,Nclusters);
%finds Cluster Centers for vectors in matrix X,
%using hierarchical clustering (i.e. NOT for least distortion).
%
%Result can be used to create a VQ object:
%vq=VQ('CodeBook',cluster(x,10));%with given training data x. 
%
%Input:
%X=         matrix with column data vectors to be clustered.
%Nclusters= desired no of clusters.
%
%Result:
%C=         cluster centers for vectors in X.
%
%Method:
%Clustering algorithm joins clusters with least distance between them 
%This tends to spread cluster centers evenly over the data vectors
%ref Duda & Hart (1973)
%Cluster distance is measured as distance between cluster centers
%(other distance measures are possible).
%NOTE: this method is rather slow, and not practical for 
%training data sets larger than about 1000 vectors.
%
%Arne Leijon 2006-04-03 tested


function C=cluster(X,Nclusters);
C=X;%start with all given vectors
nC=ones(1,size(X,2));%no of X-vectors reduced into each C-vector

while Nclusters<size(C,2)
	C2=sum(C.*C);
	d=repmat(C2',1,size(C,2))+repmat(C2,size(C,2),1)-2*C'*C;%all sq distances
	d=d+diag(repmat(+Inf,size(C,2),1));%exclude diagonal
	[dMin,iMin]=min(d);%distance to nearest other code-vector
	[d,i]=min(dMin);
	j=iMin(i);%to be joined with i
	C(:,i)=(nC(i).*C(:,i)+nC(j).*C(:,j))./(nC(i)+nC(j));%new mean vector
	nC(i)=nC(i)+nC(j);
	iKeep=(j~=[1:size(C,2)]);%all except the joined vector
	C=C(:,iKeep);
	nC=nC(iKeep);
end;
