% Gustav Eje Henter 2013-04-08

classdef KDEMM < KernelDensity
    properties (Access=public)
        Order = 1;
        Stationary = true;
        Crossval = false;
        FastApproxUpdate = true;
        UpdateAllParms = true;
        UpdateWeights = false;
    end
    methods (Access=public)
        function mm = KDEMM(varargin)
            if (nargin < 1),
                return; % Default empty KDEMM
            elseif (nargin == 1),
                arg1 = varargin{1};
                if isa(arg1,'KDEMM')
                    mm = arg1;
                    return;
                elseif isa(arg1,'KernelDensity')
                    mm.Points = arg1.Points;
                    mm.Weights = arg1.Weights;
                    mm.Kern = arg1.Kern;
                    mm.DSize = arg1.DSize;
                    mm.Order = 1;
                    return;
                end
                ws = ones(1,size(arg1,2));
            else
                arg1 = varargin{1};
                ws = varargin{2};
            end
            mm.init(arg1,ws);
        end
        
        % Training methods
        mm = init(mm,xs,ws);
        %[mm,logprobs]=train(mm,xs,nIter,minStep);
        
        % Probability computation methods
        % (These files are identical copies of those of the superclass
        % KernelDensity, in order to call KDEMM-modified protected methods
        % from within.)
        [p,logS,rhos] = prob(pD,xs);
        [lp,lrhos] = logprob(pD,xs); % Altered to account for output length
        
        % Conditional probability and log-conditional methods
        % (These files are functionally identical to those of the superclass
        % KernelDensity, in order to call KDEMM-modified protected methods
        % from within.)
        [p,logS,rhos] = conditionalProb(mm,xs,x0);
        [lp,lrhos] = conditionalLogprob(mm,xs,x0);
        
        % Data generation methods
        [xs,zs] = rand(mm,nSamples);
        [xs,zs] = conditionalRand(mm,x0,nSamples);
        
        % EM training methods
        %aState = adaptStart(mm);
        %aState = adaptAccum(mm,aState,x,gamma);
        %mm = adaptSet(mm,aState);
        
        % Get-set methods are purely inherited from KernelDensity
    end
    methods (Access=protected),
        pts = series2points(mm,xs,periodicExtension);
        xs = points2series(mm,pts,includeStart);
        
        [lp,lrhos] = singleLogprob(mm,xs);
        [lp,lrhos] = singleConditionalLogprob(mm,xs,x0);
    end
end