% Gustav Eje Henter 2013-04-08

function mm = adaptSet(mm,aState)

Q = numel(aState);
Np = numel(aState(1).sumGammaRho);

gammarhos = [aState(:).sumGammaRho];
gammarhos = spdiags(1./sum(gammarhos,2),0,Np,Np)*gammarhos;

showDebugInfo(mm,aState);

for i = 1:Q,
    if (aState(i).sumGamma > 0),
        mm(i) = setSingle(mm(i),aState(i),gammarhos(:,i)');
    end
end

%showDebugInfo(mm,aState);

function mm = setSingle(mm,aState,ws)

Dx = mm.DSize;
o = mm.Order;

numerator = [repmat(aState.sumOmega,o*Dx,1);zeros(Dx,1)];
numerator = numerator + aState.sumGamma;

if mm.UpdateAllParms,
    % Update (all) bandwidths
    mm.Kern = mm.Kern.setTaus(numerator,aState.sumSufStat);
else
    % Only update next-step bandwidths
    mm.Kern.Taus(end) = numerator(end)/aState.sumSufStat(end);
end
%mm.Kern.Taus = min(realmax,max(realmin,mm.Kern.Taus)); % Avoid infinite taus

if mm.UpdateWeights,
    mm.Weights = ws./sum(ws);
end

function showDebugInfo(mm,aState)

scaler = 1;

Q = numel(aState);

gammas = [];
taus = [];

for i = 1:Q,
    gammas = [gammas;reshape([aState(i).gammas{:}],1,[])];
    taus = [taus,mm(i).Kern.Taus];
end

[~,qs] = max(gammas,[],1);

dcolors = distinguishable_colors(Q,'w');

if (mm(1).Order < 1),
    return;
elseif (mm(1).Order < 2),
    dims = [1 2];
else
    %dims = mm(1).Order + [-1 0];
    dims = mm(1).Order + [0 1];
end

%taus,

clf;
hold on;
for s = 1:Q,
    is = find(qs == s);
    if (scaler > 0),
        stdevs = 1./sqrt(mm(s).Kern.Taus(dims));
        
        for i = is,
            rectangle('Position',[mm(s).Points(dims(1),i) - scaler*stdevs(1)/2,...
                mm(s).Points(dims(2),i) - scaler*stdevs(2)/2,...
                scaler*stdevs(1), scaler*stdevs(2)],...
                'Curvature',[1 1],'EdgeColor',dcolors(s,:));
        end
    else
        plot(mm(s).Points(dims(1),is),mm(s).Points(dims(2),is),'.',...
            'Color',dcolors(s,:));
    end
end
hold off;
drawnow;