% Gustav Eje Henter 2013-03-18

function [lp,lrhos] = singleLogprob(mm,xs)

o = mm.Order;
Np = mm.NumPoints;

crossval = mm.Crossval || (nargin < 2) || isempty(xs);
if crossval,
    % Use self-log-prob with cross-validation
    %Nx = Np - o*mm.Stationary; % Not necessary
    [lp,lrhos] = singleConditionalLogprob(mm,[],[]);
else
    Nx = size(xs,2);
    if (Nx > o),
        [lp,lrhos] = singleConditionalLogprob(mm,xs(:,o+1:end),xs(:,1:o));
    else
        lp = []; % Not enough data to compute a value
        lrhos = zeros(Np,0);
    end
end

% If stationary, include the marginal log-probability of the starting point
if mm.Stationary && (o > 0),
    if ~crossval,
        Nv = min(1+o,Nx);
        pts = mm.series2points(xs(:,1:Nv));
        [lp(1),lrhos(:,1)] = marginalLogprob(mm,pts,1:(mm.DSize*Nv));
    else
        lpz0 = mm.Kern.logprob(mm.Points,mm.Points(:,1));
        
        lpz0 = mm.addLogWeights(crossval,lpz0);
        
        lp0 = logsumexp(lpz0,1);
        lp(1) = lp0;

        lrhos(:,1) = lpz0 - lp0; % Normalize by dividing by the sum
    end
end