% Gustav Eje Henter 2012-11-30

function aState = adaptAccum(mm,aState,x,gamma)

nObj = numel(mm);

if (nargin < 4) || isempty(gamma)
    if mm(1).Crossval,
        gamma = prob(mm,[]);
    else
        gamma = prob(mm,xs);
    end
    gamma = gamma./repmat(sum(gamma,1),nObj,1);
end

for i = 1:nObj,
    aState(i) = accumSingle(mm(i),aState(i),x,gamma(i,:));
end

function aState = accumSingle(mm,aState,x,gamma)

o = mm.Order;
T = size(x,2);

aState.xs = [aState.xs x];
aState.lxs(end+1) = T;
aState.gammas{end+1} = gamma;


% Compute responsibilities
if mm.Crossval,
    [~,~,rhos] = mm.prob([]);
    
    xseg = zeros(o+1,T-o);
    for l = 1:o+1,
        xseg(l,:) = x(l:end-o-1+l);
    end
else
    [~,~,rhos] = mm.prob(x);
    
    xseg = mm.Points;
end

Nx = size(xseg,2);

% Compute component occupancies
lpzs = mm.Kern.marginalLogprob(mm.Points,xseg,[false(o,1);true]);
gmrhos = rhos.*exp(lpzs);
gmrhos = gmrhos./repmat(sum(gmrhos,1),Nx,1); % Renormalize

aState.gmrhos{end+1} = gmrhos;

aState.sumGamma = aState.sumGamma + sum(gamma);

d2ntend = sqdistance(mm.Points(o+1,:),xseg(o+1,:),1);
aState.sumSqrDev = aState.sumSqrDev + sum((d2ntend.*gmrhos)*gamma');