% Gustav Eje Henter 2012-11-07

function [p,rhos,logS] = conditionalProb(pD,xs,givenDims,bwparms)

if (nargin < 4),
    bwparms = [];
end

N = pD.NumPoints;

crossval = false;
if (nargin < 2) || isempty(xs),
    xs = []; % Compute self-log-prob later
    crossval = true;
end

if (nargin < 3) || isempty(givenDims),
    [p,rhos,logS] = prob(pD,xs,bwparms); % Assume the unconditional case
    return;
end

[lpjoint,logSjoint] = pD.Kern.logprob(pD.Points,xs,bwparms);
[lpmarg,logSmarg] = pD.Kern.marginalLogprob(pD.Points,xs,givenDims,bwparms);

%[lpzs,logS] = pD.Kern.conditionalLogprob(pD.Points,xs,givenDims,bwparms);

Nx = size(lpjoint,2);

% Incorporate component weights
wm = repmat(pD.Weights',1,Nx);
if crossval,
    wm = wm./repmat(1 - wm(1:N+1:N*N),N,1);
    wm(1:N+1:N*N) = 0;
end
logwm = log(wm);

lpjoint = lpjoint + logwm;
lpmarg = lpmarg + logwm;

% Recompute logS
logStmp = max(lpjoint,[],1);
lpjoint = lpjoint - repmat(logStmp,N,1);
logSjoint = logSjoint + logStmp;

logStmp = max(lpmarg,[],1);
lpmarg = lpmarg - repmat(logStmp,N,1);
logSmarg = logSmarg + logStmp;

logS = logSjoint - logSmarg;

if (nargout > 1),
    rhos = exp(lpjoint);
    rhos = rhos./repmat(sum(rhos,1),N,1);
end

if (nargout > 2),
    p = sum(exp(lpjoint),1)./sum(exp(lpmarg),1);
else
    p = sum(exp(lpjoint),1)./sum(exp(lpmarg),1).*exp(logS);
end