% Gustav Eje Henter 2012-11-28

classdef KernelDensity < ProbDistr
    properties (Access=public)
        Points = [];
        Weights = [];
        Kern = GaussKern();
    end
    %properties (Access=private)
    %    SelfPoints = [];
    %end
    properties (Dependent,Access=public)
        DataSize;%  size of data vectors
        NumPoints;
        Mean;%      total mean
        Variance;
    end
    methods (Access=public)
        function pD = KernelDensity(varargin)
            if (nargin < 1),
                return; % Default empty kernel density
            elseif (nargin == 1),
                arg1 = varargin{1};
                if isa(arg1,'KernelDensity')
                    pD = arg1;
                    return;
                end
                ws = ones(1,size(arg1,2));
            else
                arg1 = varargin{1};
                ws = varargin{2};
            end
            pD = pD.init(arg1,ws);
        end
        
        % Training methods
        pD = init(pD,xs,ws);
        [pD,logprobs] = train(pD,maxIter);
        %[pD,logprobs] = train(pD,xs,nIter,minStep);
        
        % Usage methods
        [xs,zs] = rand(pD,nSamples,bwparms);
        [p,logS,rhos] = prob(pD,xs,bwparms);
        [lp,logS,rhos] = logprob(pD,xs,bwparms);
        
        % Conditional methods
        [p,logS,rhos] = conditionalProb(pD,xs,givenDims,bwparms);
        [p,logS,rhos] = marginalProb(pD,xs,givenDims,bwparms);
        [xs,zs] = conditionalRand(pD,x0s,givenDims,bwparms);
        
        % Currently unimplemented
        function aState = adaptStart(pD)
            aState = [];
        end
        function aState = adaptAccum(pD,aState,obsData)
        end
        function pD = adaptSet(pD,aState)
        end
    end
    methods (Access=protected),
        [p,logS,rhos] = singleProb(pD,xs,bwparms);
        [p,logS,rhos] = singleConditionalProb(pD,xs,givenDims,bwparms);
        [p,logS,rhos] = singleMarginalProb(pD,xs,givenDims,bwparms)
    end
    methods% get/set methods
        function D = get.DataSize(pD)
            D = size(pD.Points,1);
        end
        function N = get.NumPoints(pD)
            N = size(pD.Points,2);
        end
        function M = get.Mean(pD)
            M = (pD.Points*pD.Weights')/sum(pD.Weights);
        end
        function V = get.Variance(pD)
            M = pD.Mean;
            N = pD.NumPoints;
            residuals = pD.Points - repmat(M,1,N);
            V = ((residuals.^2)*pD.Weights')/sum(pD.Weights)...
                + sum(pD.Weights)*pD.Kern.Variance;
        end
    end
end