% Gustav Eje Henter 2012-11-28

function [xs,zs] = conditionalRand(pD,x0s,givenDims,bwparms)

if (numel(pD) ~= 1),
    error('This function only works for a single object!');
end

if (nargin < 4),
    bwparms = [];
end

if (nargin < 2) || isempty(x0s),
    [xs,zs] = rand(pD,[],bwparms);
    return;
end
if (nargin < 3) || isempty(givenDims),
    [xs,zs] = rand(pD,[],bwparms); % Assume the unconditional case
    return;
end

N = pD.NumPoints;
D = pD.DataSize;
nSamples = size(x0s,2);

if ~islogical(givenDims),
    givenDims = ismember((1:D)',givenDims);
else
    givenDims = reshape(givenDims,[],1);
end

prune = false;
if (size(x0s,1) == sum(givenDims)),
    prune = true;
    xs = zeros(D,nSamples);
    xs(givenDims,:) = x0s;
else
    xs = x0s;
end

zs = zeros(1,nSamples);

[~,~,pzconds] = marginalProb(pD,xs,givenDims,bwparms);

cmf = cumsum(pzconds,1);
cmf = cmf./repmat(cmf(end,:),N,1); % Renormalize just to be sure

for t = 1:nSamples,
    zs(t) = find(rand < cmf(:,t),1,'first');
end

xs(~givenDims,:) = pD.Points(~givenDims,zs);

%xs = xs + pD.Kern.rand(nSamples,bwparm).*repmat(~givenDims,1,nSamples);
xs(~givenDims,:) = xs(~givenDims,:)...
    + pD.Kern.conditionalRand(nSamples,givenDims,bwparms);

if prune,
    xs = xs(~givenDims,:);
end