% Gustav Eje Henter 2012-11-28

function [p,logS,rhos] = marginalProb(pD,xs,givenDims,bwparms)

if (nargin < 4),
    bwparms = [];
end

if (nargin < 2) || isempty(xs),
    xs = []; % Compute self-log-prob later
    Nx = pD(1).NumPoints; % We assume all pD have the same number of points
else
    Nx = size(xs,2);
end

nD = numel(pD);

p = ones(nD,Nx);
logSall = zeros(nD,Nx);
%logS = zeros(1,Nx);
rhos = cell(nD,1);

if (nargin < 3) || isempty(givenDims),
    %givenDims = false(pD.DataSize,1); % Assume the unconditional case
    %p = ones(nD,Nx);
    for i = 1:nD,
        rhos{i} = repmat(pD(i).Weights,1,Nx);
    end
else
    for i = 1:nD,
        if (size(bwparms,2) == nD),
            bwparm = bwparms(:,i); % Different bandwidths for each model
        else
            bwparm = bwparms;
        end
        [p(i,:),logSall(i,:),rhos{i}]...
            = singleMarginalProb(pD(i),xs,givenDims,bwparm);
    end
end

if (nD == 1),
    if iscell(rhos),
        rhos = rhos{1};
    end
    logS = logSall;
else
    % Recompute logS such that the largest p-element in each column is one
    logS = max(log(p) + logSall,[],1);
    p = p.*exp(logSall - repmat(logS,nD,1));
end

if (nargout < 2),
    p = p.*repmat(exp(logS),nD,1);
end