% Gustav Eje Henter 2013-05-03

function [hmm,hmm0,lps] = MakeErgodicARHMM(initvar,order,xs,miniter,minstep)

if (nargin < 2) || isempty(order),
    order = 0;
end

if (nargin < 4) || isempty(miniter),
    miniter = 10;
end

if (nargin < 5) || isempty(minstep),
    minstep = log(1.01);
end

defaultinit = false;
if isa(initvar,'HMM'),
    mc = initvar.StateGen;
    gammas = initvar.stateOccupancy(xs);
elseif isa(initvar,'GaussMixD'),
    [mc,gammas] = initFromGaussMixD(initvar,xs);
elseif isa(initvar,'GaussD'),
    gm = GaussMixD(initvar);
    [mc,gammas] = initFromGaussMixD(gm,xs);
elseif isa(initvar,'double'),
    if isempty(initvar),
        gammas = ones(1,size(xs,2)); % Assume one single state (a Markov model)
        mc = gammas2mc(gammas);
    elseif (numel(initvar) == 1),
        defaultinit = true;
        nStates = initvar;
    else
        gammas = initvar;
        mc = gammas2mc(gammas);
    end
else
    error(['Unrecognized initialization class "' class(initvar) '."']);
end

lxs = size(xs,2);

if defaultinit,
    gd = repmat(GaussAR('Order',order,'Mean',mean(xs,2)),nStates,1);
    gd = gd.init(xs,lxs);
    
    pX = gd.prob(xs);
    gammas = pX./repmat(sum(pX,1),nStates,1);
    
    mc = gammas2mc(gammas);
else
    % Crop the beginning of xs or gammas to make the sizes match
    nX = size(xs,2);
    nG = size(gammas,2);
    if (nX > nG+order),
        xs = xs(:,end-nG-order+1:end);
        lxs = size(xs,2);
    else
        gammas = gammas(:,end-nX+order+1:end);
    end
    
    nStates = size(gammas,1);
    gd = repmat(GaussAR('Order',order,'Mean',mean(xs,2)),nStates,1);
    
    aState = adaptStart(gd);
    aState = adaptAccum(gd,aState,xs,gammas);
    gd = adaptSet(gd,aState);
end

hmm0 = ARHMM('StateGen',mc,'OutputDistr',gd);

if (nStates <= 1), % EM converges instantly if there is only one state
    miniter = 1;
    minstep = Inf;
end

[hmm,lps] = hmm0.train(xs,lxs,miniter,minstep);

 function [mc,rhos] = initFromGaussMixD(gd,xs)
 [mc,rhos] = initFromGaussMixD(gd,xs)


%nS = numel(gd.MixWeight);
 [Dx Nx] = size(xs);
 Dg = gd.DataSize;
 go = Dg/Dx-1;

 pts = make_pts(xs,go);

 rhos = gmm_rhos(gd,pts);
 mc = gammas2mc(rhos);
