% Gustav Eje Henter 2013-05-04

function [kdeh,kdeh0,lps] = MakeErgodicKDEHMM(initvar,o,xs,miniter,minstep)

if (nargin < 4) || isempty(miniter),
    miniter = 10;
end

if (nargin < 5) || isempty(minstep),
    minstep = log(1.01);
end

if isa(initvar,'HMM'),
    mc = initvar.StateGen;
    gammas = initvar.stateOccupancy(xs);
elseif isa(initvar,'GaussMixD'),
    [mc,gammas] = initFromGaussMixD(initvar,xs);
elseif isa(initvar,'GaussD'),
    gm = GaussMixD(initvar);
    [mc,gammas] = initFromGaussMixD(gm,xs);
elseif isa(initvar,'double'),
    if isempty(initvar), % Assume one single state (a Markov model)
        gammas = ones(1,size(xs,2) - o);
    else
        gammas = initvar;
    end
    mc = gammas2mc(gammas);
else
    error(['Unrecognized initialization class "' class(initvar) '."']);
end

nS = mc.nStates;

mm = KDEMM;
mm.Order = o;
if (nS <= 1),
    mm.Stationary = false;%true;
    mm.UpdateWeights = false;
else
    mm.Stationary = false;
    mm.UpdateWeights = false;%true;
end
mm = repmat(mm,nS,1);

% Crop the beginning of xs or gammas to make the sizes match
nX = size(xs,2);
nG = size(gammas,2);
if (nX > nG+o),
    xs = xs(:,end-nG-o+1:end);
else
    gammas = gammas(:,end-nX+o+1:end);
end

for i = 1:nS,
    mm(i) = init(mm(i),xs,gammas(i,:));
end

kdeh0 = KDEHMM('StateGen',mc,'OutputDistr',mm);

[kdeh,lps] = kdeh0.train([],[],miniter,minstep);

function [mc,rhos] = initFromGaussMixD(gd,xs)

%nS = numel(gd.MixWeight);
[Dx Nx] = size(xs);
Dg = gd.DataSize;
go = Dg/Dx-1;

pts = zeros(Dg,Nx-go);
for l = 0:go,
    pts(Dx*l+1:Dx*(l+1),:) = xs(:,l+1:end-go+l);
end

rhos = gmm_rhos(gd,pts);

mc = gammas2mc(rhos);