% Gustav Eje Henter 2013-06-23
% File: kdehmm_experiment3.m

function kdehmm_experiment3(datastr,ntrain,ntest,gridres,condq,...
    maxorderm,maxorderh,...
    maxstates,maxiter,nreinits,inititer,miniter,minstep,initarg,initarg2)

starttime = now;

disp(['KDE-HMM experiment started on dataset "' datastr '" at '...
    datestr(starttime,31) '...']);

% Load data

savefigs = false;
set(0,'DefaultFigureVisible','off');

load([datastr '_data']);

xv = xs(:,ntrain+1:ntrain+ntest);
xs = xs(:,1:ntrain);

outpath   = ['.' filesep datastr filesep];  % filesep = \
matsuffix = '_results';

matfile = [outpath datastr matsuffix];

% Set up plots

%% warning('off','MATLAB:HandleGraphics:noJVM');
%% warning('off','VQ:RemovedCode');

xplot = xrange(1):res1d:xrange(2);
x2s  = [xs(:,1:end-1);xs(:,2:end)];


% Train KDEs

kde10           = KernelDensity(xs);
[kde1t,lpskde1] = kde10.train(maxiter);

kde20           = KernelDensity(x2s);
[kde2t,lpskde2] = kde20.train(maxiter);


% Set up conditional probabilities (cut through 2D KDE)

givenDims = 1;

xcond = quantile(xs,condq);
ndec  = max(0,1-floor(log10(diff(xrange))));
xcond = round((10^ndec)*xcond)*(10^-ndec);

xconds = repmat(xplot,2,1);
xconds(givenDims,:) = xcond;

% Estimate Gaussian parameters

mux = mean(xs);
sigmax = std(xs,1);

lpnormt = mean(log(normpdf(xs,mux,sigmax)));
lpnormv = mean(log(normpdf(xv,mux,sigmax)));

% Train AR models

disp('AR models');

[am0,amt] = deal(repmat(ARHMM,maxorderm+1,1));
lpsamt = cell(maxorderm+1,1);
[lpsamt{:}] = deal(-Inf);
[lpamt,lpamv] = deal(zeros(maxorderm+1,1));

lpamt(1) = lpnormt;
lpamv(1) = lpnormv;

for o = 1:maxorderm,
    [amt(o+1),am0(o+1),lpsamt{o+1}] = MakeErgodicARHMM(1,o,xs,inititer,minstep);
    
    lpamt(o+1) = amt(o+1).logprob(xs)/(ntrain-o);
    lpamv(o+1) = amt(o+1).logprob(xv)/(ntest-o);
    
    disp(['p=' int2str(o) ', ns=' int2str(1)...
        ': lpt=' num2str(lpamt(o+1)) ', lpv=' num2str(lpamv(o+1))]);
end

[~,bestamo] = nanmax(lpamv);
bestam      = amt(bestamo);
bestamo     = bestamo - 1;

% Train stationary and nonstationary KDE-MMs

disp('KDE-MMs');

[mmpts,mmp0s] = deal(repmat(KDEMM,maxorderm+1,2));

lpsmmt        = cell(maxorderm+1,2);
[lpmmt,lpmmv] = deal(zeros(maxorderm+1,2));

statstrs = {'stationary','nonstationary'};
for nonstat = 0:1,
    for o = 0:maxorderm,
        mmp0s(o+1,1+nonstat).Order = o;
        mmp0s(o+1,1+nonstat).Stationary = (nonstat == 0);
        
        mmp0s(o+1,1+nonstat) = mmp0s(o+1,1+nonstat).init(xs);
        
        [mmpts(o+1,1+nonstat),lpsmmt{o+1,1+nonstat}]...
            = mmp0s(o+1,1+nonstat).train(maxiter);
        
        lpmmt(o+1,1+nonstat) = sum(mmpts(o+1,1+nonstat).logprob(xs))/(ntrain-o);
        lpmmv(o+1,1+nonstat) = sum(mmpts(o+1,1+nonstat).logprob(xv))/(ntest-o);
        
        disp(['p=' int2str(o) ', ' statstrs{1+nonstat}...
            ': lpt=' num2str(lpmmt(o+1,1+nonstat))...
            ', lpv=' num2str(lpmmv(o+1,1+nonstat))]);
    end
end

[~,bestmmos] = nanmax(lpmmv(:,1));
bestmmos     = bestmmos - 1;

[~,bestmmon] = nanmax(lpmmv(:,2));
bestmmon = bestmmon - 1;

save('c:\Hidden-Markov-KDE-HMM\ar_kde\Rate_ALL\matfile')


% Train initialization distributions (k-means or GMMs)

nss = 1:maxstates;

if (numel(initarg) < 2),
    % Use general-purpose initialization (k-means++ or GMMs)
    initorder = initarg2;
    hmmgammainit = false;
    
    initgammas = cell(maxstates,nreinits);
    if initarg, % GMM initialization
        initds = repmat(GaussMixD,maxstates,nreinits);
        initgmmlps = cell(maxstates,nreinits);
        for ns = nss,
            for r = 1:nreinits,
                [initgammas{ns,r},initds(ns,r),initgmmlps{ns,r}]...
                    = gmm_assign(ns,initorder,xs,inititer,minstep,true);
            end
        end
    else % k-means++ initialization
        initds = repmat(VQ,maxstates,nreinits);
        vqsigmas = cell(maxstates,nreinits);
        %initvqsigma = 0; % Use hard assignment (bad for KDE-HMMs)
        initvqsigma = -1; % Soft assignment with ML-estimated standard deviation
        for ns = nss,
            for r = 1:nreinits,
                [initgammas{ns,r},initds(ns,r),vqsigmas{ns,r}]...
                    = vq_assign(ns,initorder,xs,true,initvqsigma);
            end
        end
    end
else
    initgammas = initarg; % Initial gamma values have already been supplied
    hmmgammainit = initarg2; % Whether or not to initialize from (AR)-HMMs
end

% Train Gaussian HMMs

disp('HMMs');

[hmmt,hmm0,lpht,lphv,besthmm,bestho,besthns,lpsht,lpshr,lpdminh,...
    ttimeh,bestrh] = train_block(HMM,@createHMM,false,0,nss,...
    xs,initgammas,inititer,miniter,minstep,xv,matfile);

% Train AR-HMMs

disp('AR-HMMs');

[art,ar0,lpart,lparv,bestar,bestaro,bestarns,lpsart,lpsarr,lpdminar,...
    ttimear,bestrar] = train_block(ARHMM,@createARHMM,false,1:maxorderh,nss,...
    xs,initgammas,inititer,miniter,minstep,xv,matfile);


% Plot best initial data partitioning

bstates = bestar.nStates;
bo      = bestar.OutputDistr(1).Order;
% QBIC    = lparv*log(bstates)*log

gammas = initgammas{bstates,bestrar(bo,bstates)}; % Assumes nss = 1:maxstates
gammas = gammas(:,1:end-bo);
[~,qs] = nanmax(gammas,[],1);

save('c:\Hidden-Markov-KDE-HMM\ar_kde\Rate_ALL\matfile')


% Plot best AR-HMM data partitioning

bstates = bestar.nStates;
bo      = bestar.OutputDistr(1).Order;

gammas = bestar.stateOccupancy(xs);
[~,qs] = nanmax(gammas,[],1);

save('c:\Hidden-Markov-KDE-HMM\ar_kde\Rate_ALL\matfile')

return;  % stop training early; do not train KDE-HMMs

% Train KDE-HMMs

disp('KDE-HMMs');

if hmmgammainit,
    % Use alignments from best (AR-)HMMs to initialize KDE-HMMs
    kdehinitgammas = cell(maxstates,1);
    for ns = nss,
        [bestarlpatns,bestaroatns] = nanmax(lpart(:,ns)); % Ignore NaNs
        if (lpht(1,ns) >= bestarlpatns),
            kdehinitgammas{ns} = hmmt(1,ns).stateOccupancy(xs);
        else
            kdehinitgammas{ns} = art(bestaroatns,ns).stateOccupancy(xs);
        end
    end
else
    kdehinitgammas = initgammas;
end

% disp(bestarns)
nss = numel(nss);
maxorderh = bestarns;
initorder = bestaro;
nss = bestarns;

[kdeht,kdeh0,lpkdeht,lpkdehv,bestkdeh,bestkdeho,bestkdehns,...
    lpskdeht,lpskdehr,lpdminkdeh,ttimekdeh,bestarkdeh]...
    = train_block2(KDEHMM,@createKDEHMM,true,maxorderh,nss,...
    xs,kdehinitgammas,inititer,miniter,minstep,xv,matfile);


% Plot random KDE-HMM output

gammas  = bestkdeh.stateOccupancy(xs);

xkdehrt = bestkdeh.conditionalRand(xs(:,1:bestkdeho),ntrain-bestkdeho,...
    gammas(:,1)); % Start from a reasonable state, given xs context


% Save results and exit

endtime = now;

save('c:\Hidden-Markov-KDE-HMM\ar_kde\Rate_ALL\matfile')


disp(['KDE-HMM experiment successfully completed at ' datestr(endtime,31) '.']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [hmmt,hmm0,lpt,lpv,besthmm,besto,bestns,lpst,lpsr,lpdmin,...
    ttime,bestr] = train_block(hmmexample,createfun,trainempty,os,nss,...
    xs,initvars,inititer,miniter,minstep,xv,matfile)

ntrain = size(xs,2);
ntest  = size(xv,2);

no   = numel(os);  % number of array elements
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt]   = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr]   = deal(cell(no,nnss));
[lpst{:}]     = deal(-Inf);
[lpsr{:}]     = deal(repmat({-Inf},1,nreinits));
[lpt,lpv]     = deal(-Inf*ones(no,nnss));
lpdmin        = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

for o = 1:no,
    for ns = 1:nnss,
        totaliter = 0;
        begintime = clock;
        begincpu = cputime;
        for r = 1:nreinits, % Train many initial models
            [hmmttmp,hmm0tmp,lpsr{o,ns}{r}]...
                = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
            
            totaliter = totaliter + numel(lpsr{o,ns}{r});
            dmintmp = nanmin(diff(lpsr{o,ns}{r}));
            if ~isempty(dmintmp),
                lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
            end
            % Keep track of the most promising model
            if (lpsr{o,ns}{r}(end) > lpst{o,ns}(end)),
                bestr(o,ns) = r;
                hmmt(o,ns) = hmmttmp;
                hmm0(o,ns) = hmm0tmp;
                lpst{o,ns} = lpsr{o,ns}{r};
            end
        end
        
        % Perform additional training on the most promising candidate
        if trainempty,
            [hmmt(o,ns),lpstmp] = hmmt(o,ns).train([],[],miniter,minstep);
        else
            [hmmt(o,ns),lpstmp]...
                = hmmt(o,ns).train(xs,ntrain,miniter,minstep);
        end
        lpst{o,ns} = [lpst{o,ns} lpstmp];
        
        totaliter = totaliter + numel(lpstmp);
        dmintmp = nanmin(diff(lpst{o,ns}));
        if ~isempty(dmintmp),
            lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
        end
        endtime = clock; % Record time required
        endcpu = cputime;
        
        % Reset initial distribution, since we assume stationarity
        if (ns > 1),% && ~any(any(isnan(hmmt(o,ns).StateGen.TransitionProb))),
            try
                hmmt(o,ns).StateGen = hmmt(o,ns).StateGen.setStationary;
            catch err
                warning('kdehmm_experiment:setStationary',...
                    ['Could not set stationary distribution due to error "'...
                    err.message '"']);
            end
        end
        
        % Evaluate performance of stationarized models
        lpt(o,ns) = hmmt(o,ns).logprob(xs)/(ntrain-os(o));
        lpv(o,ns) = hmmt(o,ns).logprob(xv)/(ntest-os(o));
        
        disp(['p=' int2str(os(o)) ', ns=' int2str(nss(ns))...
            ': lpt=' num2str(lpt(o,ns)) ', lpv=' num2str(lpv(o,ns))...
            ', secs=' num2str(etime(endtime,begintime))]);
        
        ttime(o,ns) = (endcpu - begincpu)/totaliter; % Store time per iteration

        
%       save([matfile '_train_block']); % Save partial training results
    end
end

[~,besti]      = nanmax(lpv(:));
besthmm        = hmmt(besti);
[besto,bestns] = ind2sub(size(lpv),besti);

besto  = os(besto);
bestns = nss(bestns);

function [hmm,hmm0,lps] = createHMM(ns,o,initvar,xs,inititer,minstep)

[hmm,hmm0,lps] = MakeErgodicGaussHMM(initvar,xs,inititer,minstep);

function [ar,ar0,lps] = createARHMM(ns,o,initvar,xs,inititer,minstep)

[ar,ar0,lps] = MakeErgodicARHMM(initvar,o,xs,inititer,minstep);

function [kdeh,kdeh0,lps] = createKDEHMM(ns,o,initvar,xs,inititer,minstep)

[kdeh,kdeh0,lps] = MakeErgodicKDEHMM(initvar,o,xs,inititer,minstep);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [hmmt,hmm0,lpt,lpv,besthmm,besto,bestns,lpst,lpsr,lpdmin,...
    ttime,bestr] = train_block2(hmmexample,createfun,trainempty,os,nss,...
    xs,initvars,inititer,miniter,minstep,xv,matfile)

ntrain = size(xs,2);
ntest  = size(xv,2);

no   = numel(os);  % number of array elements
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt]   = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr]   = deal(cell(no,nnss));
[lpst{:}]     = deal(-Inf);
[lpsr{:}]     = deal(repmat({-Inf},1,nreinits));
[lpt,lpv]     = deal(-Inf*ones(no,nnss));
lpdmin        = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

for o = 1:no,
%    for ns = 1:nnss,
        ns = nnss;
        totaliter = 0;
        begintime = clock;
        begincpu = cputime;
        for r = 1:nreinits, % Train many initial models
            [hmmttmp,hmm0tmp,lpsr{o,ns}{r}]...
                = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
            
            totaliter = totaliter + numel(lpsr{o,ns}{r});
            dmintmp = nanmin(diff(lpsr{o,ns}{r}));
            if ~isempty(dmintmp),
                lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
            end
            % Keep track of the most promising model
            if (lpsr{o,ns}{r}(end) > lpst{o,ns}(end)),
                bestr(o,ns) = r;
                hmmt(o,ns) = hmmttmp;
                hmm0(o,ns) = hmm0tmp;
                lpst{o,ns} = lpsr{o,ns}{r};
            end
        end
        
        % Perform additional training on the most promising candidate
        if trainempty,
            [hmmt(o,ns),lpstmp] = hmmt(o,ns).train([],[],miniter,minstep);
        else
            [hmmt(o,ns),lpstmp]...
                = hmmt(o,ns).train(xs,ntrain,miniter,minstep);
        end
        lpst{o,ns} = [lpst{o,ns} lpstmp];
        
        totaliter = totaliter + numel(lpstmp);
        dmintmp = nanmin(diff(lpst{o,ns}));
        if ~isempty(dmintmp),
            lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
        end
        endtime = clock; % Record time required
        endcpu = cputime;
        
        % Reset initial distribution, since we assume stationarity
        if (ns > 1),% && ~any(any(isnan(hmmt(o,ns).StateGen.TransitionProb))),
            try
                hmmt(o,ns).StateGen = hmmt(o,ns).StateGen.setStationary;
            catch err
                warning('kdehmm_experiment:setStationary',...
                    ['Could not set stationary distribution due to error "'...
                    err.message '"']);
            end
        end
        
        % Evaluate performance of stationarized models
        lpt(o,ns) = hmmt(o,ns).logprob(xs)/(ntrain-os(o));
        lpv(o,ns) = hmmt(o,ns).logprob(xv)/(ntest-os(o));
        
        disp(['p=' int2str(os(o)) ', ns=' int2str(nss(ns))...
            ': lpt=' num2str(lpt(o,ns)) ', lpv=' num2str(lpv(o,ns))...
            ', secs=' num2str(etime(endtime,begintime))]);
        
        ttime(o,ns) = (endcpu - begincpu)/totaliter; % Store time per iteration

        
%       save([matfile '_train_block']); % Save partial training results
%    end % end ns =1:nss
 end  % end o=1:no

 [~,besti]      = nanmax(lpv(:));
 besthmm        = hmmt(besti);
 [besto,bestns] = ind2sub(size(lpv),besti);

 besto  = os(besto);
 bestns = nss(bestns);


