% Gustav Eje Henter 2013-06-23

function kdehmm_experiment_2(datastr,ntrain,ntest,gridres,condq,...
    maxorderm,maxorderh,...
    maxstates,maxiter,nreinits,inititer,miniter,minstep,initarg,initarg2)

starttime = now;

disp(['KDE-HMM experiment started on dataset "' datastr '" at '...
    datestr(starttime,31) '...']);

% Load data

savefigs = false;

load([datastr '_data']);
maxorderm = 2;

xv = xs(:,ntrain+1:ntrain+ntest);
xs = xs(:,1:ntrain);

outpath   = ['.' filesep datastr filesep];
matsuffix = '_results_2';

matfile = [outpath datastr matsuffix]; % ./laser_3k_phaseinit/laser_3k_phaseinit_results_2

% Set up plots

warning('off','MATLAB:HandleGraphics:noJVM');
warning('off','VQ:RemovedCode');

xplot = xrange(1):res1d:xrange(2);

x2s  = [xs(:,1:end-1);xs(:,2:end)];
%x2v = [xv(:,1:end-1);xv(:,2:end)];

%x3s = [xs(:,1:end-2);xs(:,2:end-1);xs(:,3:end)];
%x3v = [xv(:,1:end-2);xv(:,2:end-1);xv(:,3:end)];


% Train KDEs

kde10 = KernelDensity(xs);
[kde1t,lpskde1] = kde10.train(maxiter);

kde20 = KernelDensity(x2s);
[kde2t,lpskde2] = kde20.train(maxiter);


% Set up conditional probabilities (cut through 2D KDE)

givenDims = 1;

xcond = quantile(xs,condq);
ndec  = max(0,1-floor(log10(diff(xrange))));
xcond = round((10^ndec)*xcond)*(10^-ndec);

xconds = repmat(xplot,2,1);
xconds(givenDims,:) = xcond;

% Estimate Gaussian parameters

mux    = mean(xs);
sigmax = std(xs,1);

lpnormt = mean(log(normpdf(xs,mux,sigmax)));
lpnormv = mean(log(normpdf(xv,mux,sigmax)));

% Train AR models

disp('AR models');

[am0,amt] = deal(repmat(ARHMM,maxorderm+1,1));
lpsamt = cell(maxorderm+1,1);
[lpsamt{:}] = deal(-Inf);
[lpamt,lpamv] = deal(zeros(maxorderm+1,1));

lpamt(1) = lpnormt;
lpamv(1) = lpnormv;

for o = 1:maxorderm,
    [amt(o+1),am0(o+1),lpsamt{o+1}] = MakeErgodicARHMM(1,o,xs,inititer,minstep);
    
    lpamt(o+1) = amt(o+1).logprob(xs)/(ntrain-o);
    lpamv(o+1) = amt(o+1).logprob(xv)/(ntest-o);
    
    disp(['p=' int2str(o) ', ns=' int2str(1)...
        ': lpt=' num2str(lpamt(o+1)) ', lpv=' num2str(lpamv(o+1))]);
end

[~,bestamo] = nanmax(lpamv);
bestam  = amt(bestamo);
bestamo = bestamo - 1;

% Train stationary and nonstationary KDE-MMs

disp('KDE-MMs');

[mmpts,mmp0s] = deal(repmat(KDEMM,maxorderm+1,2));

lpsmmt = cell(maxorderm+1,2);
[lpmmt,lpmmv] = deal(zeros(maxorderm+1,2));

statstrs = {'stationary','nonstationary'};
for nonstat = 0:0,
    for o = 0:maxorderm,
        mmp0s(o+1,1+nonstat).Order = o;
        mmp0s(o+1,1+nonstat).Stationary = (nonstat == 0);
        
        mmp0s(o+1,1+nonstat) = mmp0s(o+1,1+nonstat).init(xs);
        
        [mmpts(o+1,1+nonstat),lpsmmt{o+1,1+nonstat}]...
            = mmp0s(o+1,1+nonstat).train(maxiter);
        
        lpmmt(o+1,1+nonstat) = sum(mmpts(o+1,1+nonstat).logprob(xs))/(ntrain-o);
        lpmmv(o+1,1+nonstat) = sum(mmpts(o+1,1+nonstat).logprob(xv))/(ntest-o);
        
        disp(['p=' int2str(o) ', ' statstrs{1+nonstat}...
            ': lpt=' num2str(lpmmt(o+1,1+nonstat))...
            ', lpv=' num2str(lpmmv(o+1,1+nonstat))]);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[~,bestmmos] = nanmax(lpmmv(:,1));
bestmmos = bestmmos - 1;

[~,bestmmon] = nanmax(lpmmv(:,2));
bestmmon = bestmmon - 1;


% Train initialization distributions (k-means or GMMs)

nss = 1:maxstates;

if (numel(initarg) < 2),
    % Use general-purpose initialization (k-means++ or GMMs)
    initorder = initarg2;
    hmmgammainit = false;
    
    initgammas = cell(maxstates,nreinits);
    if initarg, % GMM initialization
        initds = repmat(GaussMixD,maxstates,nreinits);
        initgmmlps = cell(maxstates,nreinits);
        for ns = nss,
            for r = 1:nreinits,
                [initgammas{ns,r},initds(ns,r),initgmmlps{ns,r}]...
                    = gmm_assign(ns,initorder,xs,inititer,minstep,true);
            end
        end
    else % k-means++ initialization
        initds = repmat(VQ,maxstates,nreinits);
        vqsigmas = cell(maxstates,nreinits);
        %initvqsigma = 0; % Use hard assignment (bad for KDE-HMMs)
        initvqsigma = -1; % Soft assignment with ML-estimated standard deviation
        for ns = nss,
            for r = 1:nreinits,
                [initgammas{ns,r},initds(ns,r),vqsigmas{ns,r}]...
                    = vq_assign(ns,initorder,xs,true,initvqsigma);
            end
        end
    end
else
    initgammas = initarg; % Initial gamma values have already been supplied
    hmmgammainit = initarg2; % Whether or not to initialize from (AR)-HMMs
end

% Train Gaussian HMMs

disp('HMMs');

%%%%%%%%  Begin training HMM %%%%
os = 0;
trainempty = false;
hmmexample = HMM;
createfun  = @createHMM;
initvars   = initgammas;

ntrain = size(xs,2);
ntest  = size(xv,2);

no = numel(os);
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt] = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr] = deal(cell(no,nnss));
[lpst{:}]   = deal(-Inf);
[lpsr{:}]   = deal(repmat({-Inf},1,nreinits));
[lpt,lpv]   = deal(-Inf*ones(no,nnss));
lpdmin      = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

for o = 1:no,
    for ns = 1:nnss,
        totaliter = 0;
        begintime = clock;
        begincpu = cputime;
        for r = 1:nreinits, % Train many initial models
            [hmmttmp,hmm0tmp,lpsr{o,ns}{r}]...
                = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
            
            totaliter = totaliter + numel(lpsr{o,ns}{r});
            dmintmp = nanmin(diff(lpsr{o,ns}{r}));
            if ~isempty(dmintmp),
                lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
            end
            % Keep track of the most promising model
            if (lpsr{o,ns}{r}(end) > lpst{o,ns}(end)),
                bestr(o,ns) = r;
                hmmt(o,ns) = hmmttmp;
                hmm0(o,ns) = hmm0tmp;
                lpst{o,ns} = lpsr{o,ns}{r};
            end
        end
        
        % Perform additional training on the most promising candidate
        if trainempty,
            [hmmt(o,ns),lpstmp] = hmmt(o,ns).train([],[],miniter,minstep);
        else
            [hmmt(o,ns),lpstmp]...
                = hmmt(o,ns).train(xs,ntrain,miniter,minstep);
        end
        lpst{o,ns} = [lpst{o,ns} lpstmp];
        
        totaliter = totaliter + numel(lpstmp);
        dmintmp = nanmin(diff(lpst{o,ns}));
        if ~isempty(dmintmp),
            lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
        end
        endtime = clock; % Record time required
        endcpu = cputime;
        
        % Reset initial distribution, since we assume stationarity
        if (ns > 1),% && ~any(any(isnan(hmmt(o,ns).StateGen.TransitionProb))),
            try
                hmmt(o,ns).StateGen = hmmt(o,ns).StateGen.setStationary;
            catch err
                warning('kdehmm_experiment:setStationary',...
                    ['Could not set stationary distribution due to error "'...
                    err.message '"']);
            end
        end
        
        % Evaluate performance of stationarized models
        lpt(o,ns) = hmmt(o,ns).logprob(xs)/(ntrain-os(o));
        lpv(o,ns) = hmmt(o,ns).logprob(xv)/(ntest-os(o));
        
        disp(['p=' int2str(os(o)) ', ns=' int2str(nss(ns))...
            ': lpt=' num2str(lpt(o,ns)) ', lpv=' num2str(lpv(o,ns))...
            ', secs=' num2str(etime(endtime,begintime))]);
        
        ttime(o,ns) = (endcpu - begincpu)/totaliter; % Store time per iteration
        
        % Plot progress
        clf;
        hold on;
        for r = 1:nreinits,
            plot(lpsr{o,ns}{r},'r');
        end
        plot(lpst{o,ns},'b');
        hold off;
        drawnow;
        
%        save([matfile '_train_block']); % Save partial training results
    end
end

%% [hmmt,hmm0,lpht,lphv,besthmm,bestho,besthns,lpsht,lpshr,lpdminh,...
%%    ttimeh,bestrh] = train_block(HMM,@createHMM,false,0,nss,...
%%    xs,initgammas,inititer,miniter,minstep,xv,matfile);

%%%%%  End training  %%%%%%%%%%%%%%

% Train AR-HMMs

disp('AR-HMMs');

%%%%%%%%  Begin training AR-HMM %%%%
% [art,ar0,lpart,lparv,bestar,bestaro,bestarns,lpsart,lpsarr,lpdminar,...
%    ttimear,bestrar] = train_block(ARHMM,@createARHMM,false,1:maxorderh,nss,...
%    xs,initgammas,inititer,miniter,minstep,xv,matfile);


os         = 1:maxorderh;
trainempty = false;
hmmexample = ARHMM;
createfun  = @createARHMM;
initvars   = initgammas;

ntrain = size(xs,2);
ntest  = size(xv,2);

no   = numel(os);
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt] = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr] = deal(cell(no,nnss));
[lpst{:}]   = deal(-Inf);
[lpsr{:}]   = deal(repmat({-Inf},1,nreinits));
[lpt,lpv]   = deal(-Inf*ones(no,nnss));
lpdmin      = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

for o = 1:no,
    for ns = 1:nnss,
        totaliter = 0;
        begintime = clock;
        begincpu = cputime;
        for r = 1:nreinits, % Train many initial models
            [hmmttmp,hmm0tmp,lpsr{o,ns}{r}]...
                = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
            
            totaliter = totaliter + numel(lpsr{o,ns}{r});
            dmintmp = nanmin(diff(lpsr{o,ns}{r}));
            if ~isempty(dmintmp),
                lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
            end
            % Keep track of the most promising model
            if (lpsr{o,ns}{r}(end) > lpst{o,ns}(end)),
                bestr(o,ns) = r;
                hmmt(o,ns) = hmmttmp;
                hmm0(o,ns) = hmm0tmp;
                lpst{o,ns} = lpsr{o,ns}{r};
            end
        end
        
        % Perform additional training on the most promising candidate
        if trainempty,
            [hmmt(o,ns),lpstmp] = hmmt(o,ns).train([],[],miniter,minstep);
        else
            [hmmt(o,ns),lpstmp]...
                = hmmt(o,ns).train(xs,ntrain,miniter,minstep);
        end
        lpst{o,ns} = [lpst{o,ns} lpstmp];
        
        totaliter = totaliter + numel(lpstmp);
        dmintmp = nanmin(diff(lpst{o,ns}));
        if ~isempty(dmintmp),
            lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
        end
        endtime = clock; % Record time required
        endcpu = cputime;
        
        % Reset initial distribution, since we assume stationarity
        if (ns > 1),% && ~any(any(isnan(hmmt(o,ns).StateGen.TransitionProb))),
            try
                hmmt(o,ns).StateGen = hmmt(o,ns).StateGen.setStationary;
            catch err
                warning('kdehmm_experiment:setStationary',...
                    ['Could not set stationary distribution due to error "'...
                    err.message '"']);
            end
        end
        
        % Evaluate performance of stationarized models
        lpt(o,ns) = hmmt(o,ns).logprob(xs)/(ntrain-os(o));
        lpv(o,ns) = hmmt(o,ns).logprob(xv)/(ntest-os(o));
        
        disp(['p=' int2str(os(o)) ', ns=' int2str(nss(ns))...
            ': lpt=' num2str(lpt(o,ns)) ', lpv=' num2str(lpv(o,ns))...
            ', secs=' num2str(etime(endtime,begintime))]);
        
        ttime(o,ns) = (endcpu - begincpu)/totaliter; % Store time per iteration
        
        % Plot progress
        clf;
        hold on;
        for r = 1:nreinits,
            plot(lpsr{o,ns}{r},'r');
        end
        plot(lpst{o,ns},'b');
        hold off;
        drawnow;
        
%        save([matfile '_train_block']); % Save partial training results
    end
end

%% [art,ar0,lpart,lparv,bestar,bestaro,bestarns,lpsart,lpsarr,lpdminar,...
%%    ttimear,bestrar] = train_block(ARHMM,@createARHMM,false,1:maxorderh,nss,...
%%    xs,initgammas,inititer,miniter,minstep,xv,matfile);

%%%%%% End training %%%%%%%%%%%

% Train KDE-HMMs

disp('KDE-HMMs');

if hmmgammainit,
    % Use alignments from best (AR-)HMMs to initialize KDE-HMMs
    kdehinitgammas = cell(maxstates,1);
    for ns = nss,
        [bestarlpatns,bestaroatns] = nanmax(lpart(:,ns)); % Ignore NaNs
        if (lpht(1,ns) >= bestarlpatns),
            kdehinitgammas{ns} = hmmt(1,ns).stateOccupancy(xs);
        else
            kdehinitgammas{ns} = art(bestaroatns,ns).stateOccupancy(xs);
        end
    end
else
    kdehinitgammas = initgammas;
end

%%%%%%%%  Begin training KDE-HMM %%%%
%% [kdeht,kdeh0,lpkdeht,lpkdehv,bestkdeh,bestkdeho,bestkdehns,...
%%    lpskdeht,lpskdehr,lpdminkdeh,ttimekdeh,bestarkdeh]...
%%    = train_block(KDEHMM,@createKDEHMM,true,0:maxorderh,nss,...
%%    xs,kdehinitgammas,inititer,miniter,minstep,xv,matfile);

%% function [hmmt,hmm0,lpt,lpv,besthmm,besto,bestns,lpst,lpsr,lpdmin,...
%%    ttime,bestr] = train_block(hmmexample,createfun,trainempty,os,nss,...
%%    xs,initvars,inititer,miniter,minstep,xv,matfile)


hmmexample = KDEHMM;
createfun  = @createKDEHMM;
trainempty = true;
os         = 0:maxorderh;
initvars   = kdehinitgammas;

ntrain = size(xs,2);
ntest  = size(xv,2);

no = numel(os);
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt] = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr] = deal(cell(no,nnss));
[lpst{:}]   = deal(-Inf);
[lpsr{:}]   = deal(repmat({-Inf},1,nreinits));
[lpt,lpv]   = deal(-Inf*ones(no,nnss));
lpdmin = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

for o = 1:no,
    for ns = 1:nnss,
        totaliter = 0;
        begintime = clock;
        begincpu = cputime;
        for r = 1:nreinits, % Train many initial models
            [hmmttmp,hmm0tmp,lpsr{o,ns}{r}]...
                = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
            
            totaliter = totaliter + numel(lpsr{o,ns}{r});
            dmintmp = nanmin(diff(lpsr{o,ns}{r}));
            if ~isempty(dmintmp),
                lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
            end
            % Keep track of the most promising model
            if (lpsr{o,ns}{r}(end) > lpst{o,ns}(end)),
                bestr(o,ns) = r;
                hmmt(o,ns) = hmmttmp;
                hmm0(o,ns) = hmm0tmp;
                lpst{o,ns} = lpsr{o,ns}{r};
            end
        end
        
        % Perform additional training on the most promising candidate
        if trainempty,
            [hmmt(o,ns),lpstmp] = hmmt(o,ns).train([],[],miniter,minstep);
        else
            [hmmt(o,ns),lpstmp]...
                = hmmt(o,ns).train(xs,ntrain,miniter,minstep);
        end
        lpst{o,ns} = [lpst{o,ns} lpstmp];
        
        totaliter = totaliter + numel(lpstmp);
        dmintmp = nanmin(diff(lpst{o,ns}));
        if ~isempty(dmintmp),
            lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
        end
        endtime = clock; % Record time required
        endcpu = cputime;
        
        % Reset initial distribution, since we assume stationarity
        if (ns > 1),% && ~any(any(isnan(hmmt(o,ns).StateGen.TransitionProb))),
            try
                hmmt(o,ns).StateGen = hmmt(o,ns).StateGen.setStationary;
            catch err
                warning('kdehmm_experiment:setStationary',...
                    ['Could not set stationary distribution due to error "'...
                    err.message '"']);
            end
        end
        
        % Evaluate performance of stationarized models
        lpt(o,ns) = hmmt(o,ns).logprob(xs)/(ntrain-os(o));
        lpv(o,ns) = hmmt(o,ns).logprob(xv)/(ntest-os(o));
        
        disp(['p=' int2str(os(o)) ', ns=' int2str(nss(ns))...
            ': lpt=' num2str(lpt(o,ns)) ', lpv=' num2str(lpv(o,ns))...
            ', secs=' num2str(etime(endtime,begintime))]);
        
        ttime(o,ns) = (endcpu - begincpu)/totaliter; % Store time per iteration
        
        % Plot progress
        clf;
        hold on;
        for r = 1:nreinits,
            plot(lpsr{o,ns}{r},'r');
        end
        plot(lpst{o,ns},'b');
        hold off;
        drawnow;
        
%        save([matfile '_train_block']); % Save partial training results
    end
end



endtime = now;


disp(['KDE-HMM experiment successfully completed at ' datestr(endtime,31) '.']);

