% Gustav Eje Henter 2013-06-23

function kdehmm_experiment(datastr,ntrain,ntest,gridres,condq,...
    maxorderm,maxorderh,...
    maxstates,maxiter,nreinits,inititer,miniter,minstep,initarg,initarg2)

starttime = now;

disp(['KDE-HMM experiment started on dataset "' datastr '" at '...
    datestr(starttime,31) '...']);

% Load data

savefigs = false;

load([datastr '_data']);

xv = xs(:,ntrain+1:ntrain+ntest);
xs = xs(:,1:ntrain);

outpath = ['.' filesep datastr filesep];
matsuffix = '_results';

matfile = [outpath datastr matsuffix];

% Set up plots

warning('off','MATLAB:HandleGraphics:noJVM');
warning('off','VQ:RemovedCode');

xplot = xrange(1):res1d:xrange(2);

x2s = [xs(:,1:end-1);xs(:,2:end)];
%x2v = [xv(:,1:end-1);xv(:,2:end)];

%x3s = [xs(:,1:end-2);xs(:,2:end-1);xs(:,3:end)];
%x3v = [xv(:,1:end-2);xv(:,2:end-1);xv(:,3:end)];

% Plot series

figure(1);
clf;
plot(xs);

curraxis = axis;
axis([curraxis(1:2) xrange]);

xlabel('Time index $t$');
ylabel([varname ' $x_t$']);

if savefigs,
    figname = [outpath datastr '_data_full'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot excerpt

zoomlen = diff(zoomloc);

figure(1);
axis([zoomloc xrange]);

if savefigs,
    figname = [outpath datastr '_data_zoomed'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot scatter

figure(1);
clf;
plot(x2s(1,:),x2s(2,:),'.');

axis([xrange xrange]);

xlabel('Current value $x_t$');
ylabel('Subsequent value $x_{t+1}$');

if savefigs,
    figname = [outpath datastr '_2d_scatter'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Train KDEs

kde10 = KernelDensity(xs);
[kde1t,lpskde1] = kde10.train(maxiter);

kde20 = KernelDensity(x2s);
[kde2t,lpskde2] = kde20.train(maxiter);

% Plot 1D KDE

figure(1);
clf;
plot(xplot,kde1t.prob(xplot));

curraxis = axis;
axis([xrange curraxis(3:4)]);

xlabel('Data series value $x_t$');
ylabel('KDE PDF $f_{X_t}(x_t)$');

if savefigs,
    figname = [outpath datastr '_1d_pdf'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot 2D KDE

p2mat = probmat(kde2t,xrange,gridres);

figure(1);
clf;
imagesc(xrange,xrange,p2mat);
axis xy;

axis([xrange xrange]);

xlabel('Current value $x_t$');
ylabel('Subsequent value $x_{t+1}$');

if savefigs,
    figname = [outpath datastr '_2d_heatmap'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Overlay scatter plot on 2D KDE

figure(1);
hold on;
plot(x2s(1,:),x2s(2,:),'r.');
hold off;

axis([xrange xrange]);

if savefigs,
    figname = [outpath datastr '_2d_heatmap_scatter'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Set up conditional probabilities (cut through 2D KDE)

givenDims = 1;

xcond = quantile(xs,condq);
ndec = max(0,1-floor(log10(diff(xrange))));
xcond = round((10^ndec)*xcond)*(10^-ndec);

xconds = repmat(xplot,2,1);
xconds(givenDims,:) = xcond;

% Plot conditioning cut

figure(1);
clf;
imagesc(xrange,xrange,p2mat);
axis xy;

hold on;
plot([max(xconds(1,1),xrange(1)) min(xconds(1,end),xrange(2))],...
    [max(xconds(2,1),xrange(1)) min(xconds(2,end),xrange(2))],'r-');
hold off;

axis([xrange xrange]);

xlabel('Given value $x_{t-1}$');
ylabel('Subsequent value $x_t$');

if savefigs,
    figname = [outpath datastr '_cond_heatmap'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

clear p2mat; % Not used later on and consumes a lot of space

% Plot conditional distribution along cut

x2cprobs = kde2t.conditionalProb(xconds,givenDims);

figure(1);
clf;
plot(xplot,x2cprobs);
disp('xplot')
disp(xplot)
disp('x2cprobs')
disp(x2cprobs)

prob_dens = x2cprobs;
x_values  = xplot;
dlmwrite('c:\temp\prob_dens.dat',prob_dens,'delimiter',' ')
dlmwrite('c:\temp\x_values.dat',x_values,'delimiter',' ')

curraxis = axis;
axis([xrange curraxis(3:4)]);

xlabel('Subsequent value $x_t$');
ylabel(['Conditional PDF $f_{X_t \mid X_{t-1}}(x_t \mid '...
    num2str(xcond) ')$']);

if savefigs,
    figname = [outpath datastr '_cond_pdf'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Estimate Gaussian parameters

mux = mean(xs);
sigmax = std(xs,1);

lpnormt = mean(log(normpdf(xs,mux,sigmax)));
lpnormv = mean(log(normpdf(xv,mux,sigmax)));

% Train AR models

disp('AR models');

[am0,amt] = deal(repmat(ARHMM,maxorderm+1,1));
lpsamt = cell(maxorderm+1,1);
[lpsamt{:}] = deal(-Inf);
[lpamt,lpamv] = deal(zeros(maxorderm+1,1));

lpamt(1) = lpnormt;
lpamv(1) = lpnormv;

for o = 1:maxorderm,
    [amt(o+1),am0(o+1),lpsamt{o+1}] = MakeErgodicARHMM(1,o,xs,inititer,minstep);
    
    lpamt(o+1) = amt(o+1).logprob(xs)/(ntrain-o);
    lpamv(o+1) = amt(o+1).logprob(xv)/(ntest-o);
    
    disp(['p=' int2str(o) ', ns=' int2str(1)...
        ': lpt=' num2str(lpamt(o+1)) ', lpv=' num2str(lpamv(o+1))]);
end

[~,bestamo] = nanmax(lpamv);
bestam = amt(bestamo);
bestamo = bestamo - 1;

% Train stationary and nonstationary KDE-MMs

disp('KDE-MMs');

[mmpts,mmp0s] = deal(repmat(KDEMM,maxorderm+1,2));

lpsmmt = cell(maxorderm+1,2);
[lpmmt,lpmmv] = deal(zeros(maxorderm+1,2));

statstrs = {'stationary','nonstationary'};
for nonstat = 0:1,
    for o = 0:maxorderm,
        mmp0s(o+1,1+nonstat).Order = o;
        mmp0s(o+1,1+nonstat).Stationary = (nonstat == 0);
        
        mmp0s(o+1,1+nonstat) = mmp0s(o+1,1+nonstat).init(xs);
        
        [mmpts(o+1,1+nonstat),lpsmmt{o+1,1+nonstat}]...
            = mmp0s(o+1,1+nonstat).train(maxiter);
        
        lpmmt(o+1,1+nonstat) = sum(mmpts(o+1,1+nonstat).logprob(xs))/(ntrain-o);
        lpmmv(o+1,1+nonstat) = sum(mmpts(o+1,1+nonstat).logprob(xv))/(ntest-o);
        
        disp(['p=' int2str(o) ', ' statstrs{1+nonstat}...
            ': lpt=' num2str(lpmmt(o+1,1+nonstat))...
            ', lpv=' num2str(lpmmv(o+1,1+nonstat))]);
    end
end

[~,bestmmos] = nanmax(lpmmv(:,1));
bestmmos = bestmmos - 1;

[~,bestmmon] = nanmax(lpmmv(:,2));
bestmmon = bestmmon - 1;

save(matfile); % Save results

% Plot Markov model training set log-likelihoods

%mmstyles = {'-.','-','--',':'};
mmstyles = {'-','-','-','-'};
dcolors = distinguishable_colors(4,{'w','k'});

figure(1);
clf;

legendstrs = {};

hold on;
plot(0:maxorderm,lpmmt(:,1),mmstyles{1},'Color',dcolors(1,:));
legendstrs{end+1} = 'KDE-MM';

plot(0:maxorderm,lpmmt(:,2),mmstyles{2},'Color',dcolors(2,:));
legendstrs{end+1} = 'KDE-HMM $M{=}1$';

plot(0:maxorderm,lpamt(:),mmstyles{3},'Color',dcolors(3,:));
legendstrs{end+1} = 'AR model';

plot(bestmmos,lpmmt(bestmmos+1,1),'*','Color',dcolors(1,:));
plot(bestmmon,lpmmt(bestmmon+1,2),'*','Color',dcolors(2,:));
plot(bestamo,lpamt(bestamo+1,1),'*','Color',dcolors(3,:));

xlabel('Markov model order $p$');
ylabel('Training set per-sample log-likelihood');

legendh = legend(legendstrs{:},'Location','East');

if savefigs,
    figname = [outpath datastr '_mm_lp_train'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot Markov model test set log-probabilities

figure(1);
clf;

legendstrs = {};

hold on;
plot(0:maxorderm,lpmmv(:,1),mmstyles{1},'Color',dcolors(1,:));
legendstrs{end+1} = 'KDE-MM';

plot(0:maxorderm,lpmmv(:,2),mmstyles{2},'Color',dcolors(2,:));
legendstrs{end+1} = 'KDE-HMM $M{=}1$';

plot(0:maxorderm,lpamv(:),mmstyles{3},'Color',dcolors(3,:));
legendstrs{end+1} = 'AR model';

plot(bestmmos,lpmmv(bestmmos+1,1),'*','Color',dcolors(1,:));
plot(bestmmon,lpmmv(bestmmon+1,2),'*','Color',dcolors(2,:));
plot(bestamo,lpamv(bestamo+1,1),'*','Color',dcolors(3,:));

xlabel('Markov model order $p$');
ylabel('Test set per-sample log-probability');

legendh = legend(legendstrs{:},'Location','East');

if savefigs,
    figname = [outpath datastr '_mm_lp_test'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot Markov model training vs. test set logprob gap

figure(1);
clf;

legendstrs = {};

hold on;
plot(0:maxorderm,lpmmt(:,1) - lpmmv(:,1),mmstyles{1},'Color',dcolors(1,:));
legendstrs{end+1} = 'KDE-MM';

plot(0:maxorderm,lpmmt(:,2) - lpmmv(:,2),mmstyles{2},'Color',dcolors(2,:));
legendstrs{end+1} = 'KDE-HMM $M{=}1$';

plot(0:maxorderm,lpamt(:) - lpamv(:),mmstyles{3},'Color',dcolors(3,:));
legendstrs{end+1} = 'AR model';

plot(bestmmos,lpmmt(bestmmos+1,1) - lpmmv(bestmmos+1,1),'*','Color',dcolors(1,:));
plot(bestmmon,lpmmt(bestmmon+1,2) - lpmmv(bestmmon+1,2),'*','Color',dcolors(2,:));
plot(bestamo,lpamt(bestamo+1,1) - lpamv(bestamo+1,1),'*','Color',dcolors(3,:));

xlabel('Markov model order $p$');
ylabel('Training and test set log-probability gap');

legendh = legend(legendstrs{:},'Location','East');

if savefigs,
    figname = [outpath datastr '_mm_lp_gap'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Train initialization distributions (k-means or GMMs)

nss = 1:maxstates;

if (numel(initarg) < 2),
    % Use general-purpose initialization (k-means++ or GMMs)
    initorder = initarg2;
    hmmgammainit = false;
    
    initgammas = cell(maxstates,nreinits);
    if initarg, % GMM initialization
        initds = repmat(GaussMixD,maxstates,nreinits);
        initgmmlps = cell(maxstates,nreinits);
        for ns = nss,
            for r = 1:nreinits,
                [initgammas{ns,r},initds(ns,r),initgmmlps{ns,r}]...
                    = gmm_assign(ns,initorder,xs,inititer,minstep,true);
            end
        end
    else % k-means++ initialization
        initds = repmat(VQ,maxstates,nreinits);
        vqsigmas = cell(maxstates,nreinits);
        %initvqsigma = 0; % Use hard assignment (bad for KDE-HMMs)
        initvqsigma = -1; % Soft assignment with ML-estimated standard deviation
        for ns = nss,
            for r = 1:nreinits,
                [initgammas{ns,r},initds(ns,r),vqsigmas{ns,r}]...
                    = vq_assign(ns,initorder,xs,true,initvqsigma);
            end
        end
    end
else
    initgammas = initarg; % Initial gamma values have already been supplied
    hmmgammainit = initarg2; % Whether or not to initialize from (AR)-HMMs
end

% Train Gaussian HMMs

disp('HMMs');

[hmmt,hmm0,lpht,lphv,besthmm,bestho,besthns,lpsht,lpshr,lpdminh,...
    ttimeh,bestrh] = train_block(HMM,@createHMM,false,0,nss,...
    xs,initgammas,inititer,miniter,minstep,xv,matfile);

% Train AR-HMMs

disp('AR-HMMs');

[art,ar0,lpart,lparv,bestar,bestaro,bestarns,lpsart,lpsarr,lpdminar,...
    ttimear,bestrar] = train_block(ARHMM,@createARHMM,false,1:maxorderh,nss,...
    xs,initgammas,inititer,miniter,minstep,xv,matfile);

% Plot regular HMM logprobs

styleorder = {':','--','-','-.'};

dcolors = distinguishable_colors(maxorderh+3,{'w','k'});

figure(1);
clf;

plot(nss,lphv,'-','Color',dcolors(1,:));
legendstrs = {'Gaussian HMM'};

hold on;
for o = 1:maxorderh,
    plot(nss,lparv(o,:),'-','Color',dcolors(o+2,:));
    legendstrs{end+1} = ['AR-HMM $p{=}' int2str(o) '$'];
end

for o = 0:maxorderh,
    plot([1 maxstates],lpmmv(o+1,1)*[1 1],...
        styleorder{mod(o,numel(styleorder))+1},'Color',dcolors(2,:));
    legendstrs{end+1} = ['KDE-MM $p{=}' int2str(o) '$'];
end

plot(1,lpnormv,'+','Color',dcolors(1,:));
for o = 1:maxorderh,
    plot(1,lparv(o,1),'+','Color',dcolors(o+2,:));
end

plot(bestarns,lparv(bestaro,bestarns),'*','Color',dcolors(bestaro+2,:));
hold off;

xlabel('Number of HMM states $M$');
ylabel('Held-out set per-sample log-probability');

legendh = legend(legendstrs{:},'Location','SouthEast');

if savefigs,
    figname = [outpath datastr '_hmm_lp_partial'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot random AR output

figure(1);
clf;
plot(bestam.rand(ntrain));

axis([0 zoomlen xrange]);

xlabel('Time index $t$');
ylabel('Output value $x_t$');

if savefigs,
    figname = [outpath datastr '_randout_ar' int2str(bestamo)];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot random HMM output

figure(1);
clf;
plot(besthmm.rand(ntrain));

axis([0 zoomlen xrange]);

xlabel('Time index $t$');
ylabel('Output value $x_t$');

if savefigs,
    figname = [outpath datastr '_randout_hmm'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot random AR-HMM output; sample from best AR-HMM

figure(1);
clf;
plot(bestar.rand(ntrain));

axis([0 zoomlen xrange]);

xlabel('Time index $t$');
ylabel('Output value $x_t$');

if savefigs,
    figname = [outpath datastr '_randout_ar' int2str(bestaro) '-hmm'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot random KDE-MM output

xmmrt = mmpts(bestmmon+1,2).conditionalRand(xs(1:bestmmon),ntrain-bestmmon);

figure(1);
clf;
plot(xmmrt);

%curraxis = axis;
%axis([curraxis(1:2) xrange]);
axis([0 zoomlen xrange]);

xlabel('Time index $t$');
ylabel('Output value $x_t$');

if savefigs,
    figname = [outpath datastr '_randout_kde' int2str(bestmmon) '-mm'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot conditional distribution along cut

 xhcprobs = besthmm.conditionalProb(xconds,givenDims);

 figure(1);
 clf;
 plot(xplot,x2cprobs);

 curraxis = axis;
 axis([xrange curraxis(3:4)]);

 xlabel('Subsequent value $x_t$');
 ylabel(['Conditional PDF $f_{X_t \mid X_{t-1}}(x_t \mid '...
    num2str(xcond+5) ')$']);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot best initial data partitioning

bstates = bestar.nStates;
bo = bestar.OutputDistr(1).Order;

gammas = initgammas{bstates,bestrar(bo,bstates)}; % Assumes nss = 1:maxstates
gammas = gammas(:,1:end-bo);
[~,qs] = nanmax(gammas,[],1);

dcolors = distinguishable_colors(bstates,'w');

figure(1);
clf;
hold on;
for s = 1:bstates,
    is = find(qs == s);
    plot(xs(is+bo-1),xs(is+bo),'.','Color',dcolors(s,:));
end;
hold off;

axis([xrange xrange]);

xlabel('First context $x_{t-2}$');
ylabel('Second context $x_{t-1}$');

if savefigs,
    figname = [outpath datastr '_partition_initial'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

save(matfile); % Save results

% Plot best AR-HMM data partitioning

bstates = bestar.nStates;
bo = bestar.OutputDistr(1).Order;

gammas = bestar.stateOccupancy(xs);
[~,qs] = nanmax(gammas,[],1);

dcolors = distinguishable_colors(bstates,'w');

figure(1);
clf;
hold on;
for s = 1:bstates,
    is = find(qs == s);
    plot(xs(is+bo-1),xs(is+bo),'.','Color',dcolors(s,:));
end;
hold off;

axis([xrange xrange]);

xlabel('First context $x_{t-2}$');
ylabel('Second context $x_{t-1}$');

if savefigs,
    figname = [outpath datastr '_partition_ar' int2str(bestaro) '-hmm'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

save(matfile); % Save results

% Train KDE-HMMs

disp('KDE-HMMs');

if hmmgammainit,
    % Use alignments from best (AR-)HMMs to initialize KDE-HMMs
    kdehinitgammas = cell(maxstates,1);
    for ns = nss,
        [bestarlpatns,bestaroatns] = nanmax(lpart(:,ns)); % Ignore NaNs
        if (lpht(1,ns) >= bestarlpatns),
            kdehinitgammas{ns} = hmmt(1,ns).stateOccupancy(xs);
        else
            kdehinitgammas{ns} = art(bestaroatns,ns).stateOccupancy(xs);
        end
    end
else
    kdehinitgammas = initgammas;
end

[kdeht,kdeh0,lpkdeht,lpkdehv,bestkdeh,bestkdeho,bestkdehns,...
    lpskdeht,lpskdehr,lpdminkdeh,ttimekdeh,bestarkdeh]...
    = train_block(KDEHMM,@createKDEHMM,true,maxorderh,nss,...
    xs,kdehinitgammas,inititer,miniter,minstep,xv,matfile);
    
disp(kdeht)    

% Plot AR-HMM and KDE-HMM training set logprobs

dcolors = distinguishable_colors(maxorderh+1,{'w','k'});
hmmstyle = '--';
kdestyle = '-';

figure(1);
clf;

plot(nss,lpht,hmmstyle,'Color',dcolors(1,:));
legendstrs = {'Gaussian HMM'};

hold on;
for o = 1:maxorderh,
    plot(nss,lpart(o,:),hmmstyle,'Color',dcolors(o+1,:));
    legendstrs{end+1} = ['AR-HMM $p{=}' int2str(o) '$'];
end

for o = 0:maxorderh,
    plot(nss,lpkdeht(o+1,:),kdestyle,'Color',dcolors(o+1,:));
    legendstrs{end+1} = ['KDE-HMM $p{=}' int2str(o) '$'];
end

plot(1,lpht(1),'+','Color',dcolors(1,:));
plot(1,lpmmt(1,1),'+','Color',dcolors(1,:));
for o = 1:maxorderh,
    plot(1,lpart(o,1),'+','Color',dcolors(o+1,:));
    plot(1,lpmmt(o+1,1),'+','Color',dcolors(o+1,:));
end

plot(bestarns,lpart(bestaro,bestarns),'*','Color',dcolors(bestaro+1,:));
plot(bestkdehns,lpkdeht(bestkdeho+1,bestkdehns),...
    '*','Color',dcolors(bestkdeho+1,:));
hold off;

xlabel('Number of HMM states $M$');
ylabel('Training set per-sample log-likelihood');

legendh = legend(legendstrs{:},'Location','SouthEast');

if savefigs,
    figname = [outpath datastr '_hmm_lp_train'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot AR-HMM and KDE-HMM test set logprobs

figure(1);
clf;

plot(nss,lphv,hmmstyle,'Color',dcolors(1,:));
legendstrs = {'Gaussian HMM'};

hold on;
for o = 1:maxorderh,
    plot(nss,lparv(o,:),hmmstyle,'Color',dcolors(o+1,:));
    legendstrs{end+1} = ['AR-HMM $p{=}' int2str(o) '$'];
end

for o = 0:maxorderh,
    plot(nss,lpkdehv(o+1,:),kdestyle,'Color',dcolors(o+1,:));
    legendstrs{end+1} = ['KDE-HMM $p{=}' int2str(o) '$'];
end

plot(1,lpnormv,'+','Color',dcolors(1,:));
plot(1,lpmmv(1,1),'+','Color',dcolors(1,:));
for o = 1:maxorderh,
    plot(1,lparv(o,1),'+','Color',dcolors(o+1,:));
    plot(1,lpmmv(o+1,1),'+','Color',dcolors(o+1,:));
end

plot(bestarns,lparv(bestaro,bestarns),'*','Color',dcolors(bestaro+1,:));
plot(bestkdehns,lpkdehv(bestkdeho+1,bestkdehns),...
    '*','Color',dcolors(bestkdeho+1,:));
hold off;

xlabel('Number of HMM states $M$');
ylabel('Held-out set per-sample log-probability');

legendh = legend(legendstrs{:},'Location','SouthEast');

if savefigs,
    figname = [outpath datastr '_hmm_lp_test'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot AR-HMM and KDE-HMM training vs. test set logprob gap

figure(1);
clf;

plot(nss,lpht - lphv,hmmstyle,'Color',dcolors(1,:));
legendstrs = {'Gaussian HMM'};

hold on;
for o = 1:maxorderh,
    plot(nss,lpart(o,:) - lparv(o,:),hmmstyle,'Color',dcolors(o+1,:));
    legendstrs{end+1} = ['AR-HMM $p{=}' int2str(o) '$'];
end

for o = 0:maxorderh,
    plot(nss,lpkdeht(o+1,:) - lpkdehv(o+1,:),kdestyle,'Color',dcolors(o+1,:));
    legendstrs{end+1} = ['KDE-HMM $p{=}' int2str(o) '$'];
end

plot(1,lpht(1) - lphv(1),'+','Color',dcolors(1,:));
plot(1,lpmmt(1,1) - lpmmv(1,1),'+','Color',dcolors(1,:));
for o = 1:maxorderh,
    plot(1,lpart(o,1) - lparv(o,1),'+','Color',dcolors(o+1,:));
    plot(1,lpmmt(o+1,1) - lpmmv(o+1,1),'+','Color',dcolors(o+1,:));
end

plot(bestarns,lpart(bestaro,bestarns)...
    - lparv(bestaro,bestarns),'*','Color',dcolors(bestaro+1,:));
plot(bestkdehns,lpkdeht(bestkdeho+1,bestkdehns)...
    - lpkdehv(bestkdeho+1,bestkdehns),'*','Color',dcolors(bestkdeho+1,:));
hold off;

xlabel('Number of HMM states $M$');
ylabel('Training and test set log-probability gap');

legendh = legend(legendstrs{:},'Location','NorthWest');

if savefigs,
    figname = [outpath datastr '_hmm_lp_gap'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot HMM training times

figure(1);
clf;

ndec = floor(log10(maxstates));
maxsplot = ceil((10^-ndec)*maxstates)*(10^ndec);

filter1 = (nss > 1);

loglog(nss(filter1),ttimeh(filter1),hmmstyle,'Color',dcolors(1,:));
legendstrs = {'Gaussian HMM'};

hold on;
for o = 1:maxorderh,
    loglog(nss(filter1),ttimear(o,filter1),hmmstyle,'Color',dcolors(o+1,:));
    legendstrs{end+1} = ['AR-HMM $p{=}' int2str(o) '$'];
end

for o = 0:maxorderh,
    loglog(nss,ttimekdeh(o+1,:),kdestyle,'Color',dcolors(o+1,:));
    legendstrs{end+1} = ['KDE-HMM $p{=}' int2str(o) '$'];
end
hold off;

curraxis = axis;
axis([curraxis(1) maxsplot curraxis(3:4)]);

xlabel('Number of HMM states $M$');
ylabel('Time per training iteration (seconds)');

%legendh = legend(legendstrs{:},'Location','East'); % Feels redundant

if savefigs,
    figname = [outpath datastr '_hmm_times'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Plot random KDE-HMM output; plot best kde-hmm

gammas = bestkdeh.stateOccupancy(xs);

xkdehrt = bestkdeh.conditionalRand(xs(:,1:bestkdeho),ntrain-bestkdeho,...
    gammas(:,1)); % Start from a reasonable state, given xs context

figure(1);
clf;
plot(xkdehrt);

%curraxis = axis;
%axis([curraxis(1:2) xrange]);
axis([0 zoomlen xrange]);

xlabel('Time index $t$');
ylabel('Output value $x_t$');

if savefigs,
    figname = [outpath datastr '_randout_kde' int2str(bestkdeho) '-hmm'];
    
    saveas(gcf,figname,'fig');
    fig2ieeetran_fm(figname);
end

% Save results and exit

endtime = now;

save(matfile);

disp(['KDE-HMM experiment successfully completed at ' datestr(endtime,31) '.']);

function [hmmt,hmm0,lpt,lpv,besthmm,besto,bestns,lpst,lpsr,lpdmin,...
    ttime,bestr] = train_block(hmmexample,createfun,trainempty,os,nss,...
    xs,initvars,inititer,miniter,minstep,xv,matfile)

ntrain = size(xs,2);
ntest = size(xv,2);

no = numel(os);
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt] = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr] = deal(cell(no,nnss));
[lpst{:}]   = deal(-Inf);
[lpsr{:}]   = deal(repmat({-Inf},1,nreinits));
[lpt,lpv]   = deal(-Inf*ones(no,nnss));
lpdmin      = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

for o = 1:no,
    for ns = 1:nnss,
        totaliter = 0;
        begintime = clock;
        begincpu = cputime;
        for r = 1:nreinits, % Train many initial models
            [hmmttmp,hmm0tmp,lpsr{o,ns}{r}]...
                = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
            
            totaliter = totaliter + numel(lpsr{o,ns}{r});
            dmintmp = nanmin(diff(lpsr{o,ns}{r}));
            if ~isempty(dmintmp),
                lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
            end
            % Keep track of the most promising model
            if (lpsr{o,ns}{r}(end) > lpst{o,ns}(end)),
                bestr(o,ns) = r;
                hmmt(o,ns) = hmmttmp;
                hmm0(o,ns) = hmm0tmp;
                lpst{o,ns} = lpsr{o,ns}{r};
            end
        end
        
        % Perform additional training on the most promising candidate
        if trainempty,
            [hmmt(o,ns),lpstmp] = hmmt(o,ns).train([],[],miniter,minstep);
        else
            [hmmt(o,ns),lpstmp]...
                = hmmt(o,ns).train(xs,ntrain,miniter,minstep);
        end
        lpst{o,ns} = [lpst{o,ns} lpstmp];
        
        totaliter = totaliter + numel(lpstmp);
        dmintmp = nanmin(diff(lpst{o,ns}));
        if ~isempty(dmintmp),
            lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
        end
        endtime = clock; % Record time required
        endcpu = cputime;
        
        % Reset initial distribution, since we assume stationarity
        if (ns > 1),% && ~any(any(isnan(hmmt(o,ns).StateGen.TransitionProb))),
            try
                hmmt(o,ns).StateGen = hmmt(o,ns).StateGen.setStationary;
            catch err
                warning('kdehmm_experiment:setStationary',...
                    ['Could not set stationary distribution due to error "'...
                    err.message '"']);
            end
        end
        
        % Evaluate performance of stationarized models
        lpt(o,ns) = hmmt(o,ns).logprob(xs)/(ntrain-os(o));
        lpv(o,ns) = hmmt(o,ns).logprob(xv)/(ntest-os(o));
        
        disp(['p=' int2str(os(o)) ', ns=' int2str(nss(ns))...
            ': lpt=' num2str(lpt(o,ns)) ', lpv=' num2str(lpv(o,ns))...
            ', secs=' num2str(etime(endtime,begintime))]);
        
        ttime(o,ns) = (endcpu - begincpu)/totaliter; % Store time per iteration
        
        % Plot progress
        clf;
        hold on;
        for r = 1:nreinits,
            plot(lpsr{o,ns}{r},'r');
        end
        plot(lpst{o,ns},'b');
        hold off;
        drawnow;
        
%        save([matfile '_train_block']); % Save partial training results
        save([matfile_new]); % Save partial training results        
    end
end

[~,besti] = nanmax(lpv(:));
besthmm = hmmt(besti);
[besto,bestns] = ind2sub(size(lpv),besti);

besto = os(besto);
bestns = nss(bestns);

function [hmm,hmm0,lps] = createHMM(ns,o,initvar,xs,inititer,minstep)

[hmm,hmm0,lps] = MakeErgodicGaussHMM(initvar,xs,inititer,minstep);

function [ar,ar0,lps] = createARHMM(ns,o,initvar,xs,inititer,minstep)

[ar,ar0,lps] = MakeErgodicARHMM(initvar,o,xs,inititer,minstep);

function [kdeh,kdeh0,lps] = createKDEHMM(ns,o,initvar,xs,inititer,minstep)

[kdeh,kdeh0,lps] = MakeErgodicKDEHMM(initvar,o,xs,inititer,minstep);