% Gustav Eje Henter 2013-06-26

function kdehmm_learning_experiment(outpath,ntrains,ntests,ntrials,...
    transp,arc,nmult,sep,miniter,minstep)

starttime = now;

disp(['KDE-HMM synthetic learning experiment started at '...
    datestr(starttime,31) '...']);

if (numel(ntests) == 1),
    ntests = ntests*ones(size(ntrains));
end

warning('off','MATLAB:HandleGraphics:noJVM');
warning('off','VQ:RemovedCode');

set(findResource(),'ClusterSize',8);
matlabpool(8);

%sub_experiment([outpath filesep 'simple_gauss'],ntrains,ntests,ntrials,...
%    @simple_gauss_ar_rand,transp,arc,nmult,sep,miniter,minstep);

sub_experiment([outpath filesep 'simple_gmm'],ntrains,ntests,ntrials,...
    @simple_gmm_ar_rand,transp,arc,nmult,sep,miniter,minstep);

matlabpool close;

endtime = now;

disp(['KDE-HMM synthetic learning experiment successfully completed at '...
    datestr(endtime,31) '.']);

function [iid_lp,ar_lp,arh_lp,h_lp,kde_lp,kdem_lp,kdeh0_lp,kdeh1_lp]...
    = sub_experiment(figname,ntrains,ntests,ntrials,...
    randcommand,transp,arc,nmult,sep,miniter,minstep)

ntps = numel(ntrains);

[iid_lp,ar_lp,arh_lp,h_lp,kde_lp,kdem_lp,kdeh0_lp,kdeh1_lp]...
    = deal(zeros(ntrials,ntps));

%[iid_m,ar_m,arh_m,h_m,kde_m,kdem_m,kdeh0_m,kdeh1_m,...
%    iid_std,ar_std,arh_std,h_std,kde_std,kdem_std,kdeh0_std,kdeh1_std]...
%    = deal(zeros(1,ntps));

for ti = 1:ntps,
    ntrain = ntrains(ti);
    ntest = ntests(ti);
    
    %for tr = 1:ntrials,
    parfor tr = 1:ntrials,
        disp(['Now using ' int2str(ntrain) ' datapoints (trial '...
            int2str(tr) ')...']);
        
        xs = randcommand(transp,arc,nmult,sep,ntrain);
        xv = randcommand(transp,arc,nmult,sep,ntest);
        
        absdeltas = abs(diff(xs));
        
        gammas = [0.5 (absdeltas <= median(absdeltas))];
        gammas = [gammas;1-gammas];
        
        gamma1 = ones(1,size(gammas,2));
        
        % IID
        mu = mean(xs);
        sigma = std(xs,1);
        iid_lp(tr,ti) = mean(log(normpdf(xv,mu,sigma)));
        
        % AR
        armodel = MakeErgodicARHMM(gamma1,1,xs,miniter,minstep);
        armodel = stationarize(armodel);
        ar_lp(tr,ti) = armodel.logprob(xv)/(ntest - 1);
        
        % AR-HMM
        arhmm = MakeErgodicARHMM(gammas,1,xs,miniter,minstep);
        arhmm = stationarize(arhmm);
        arh_lp(tr,ti) = arhmm.logprob(xv)/(ntest - 1);
        
        % HMM
        h = MakeErgodicGaussHMM(gammas,xs,miniter,minstep);
        h = stationarize(h);
        h_lp(tr,ti) = h.logprob(xv)/(ntest - 1);
        
        % KDE
        kde = MakeErgodicKDEHMM(gamma1,0,xs,miniter,minstep);
        kde = stationarize(kde);
        kde_lp(tr,ti) = kde.logprob(xv)/ntest;
        
        % KDE-MM
        kdem = MakeErgodicKDEHMM(gamma1,1,xs,miniter,minstep);
        kdem = stationarize(kdem);
        kdem_lp(tr,ti) = kdem.logprob(xv)/(ntest - 1);
        
        % KDE-HMM (p=0)
        kdeh0 = MakeErgodicKDEHMM(gammas,0,xs,miniter,minstep);
        kdeh0 = stationarize(kdeh0);
        kdeh0_lp(tr,ti) = kdeh0.logprob(xv)/ntest;
        
        % KDE-HMM (p=1)
        kdeh1 = MakeErgodicKDEHMM(gammas,1,xs,miniter,minstep);
        kdeh1 = stationarize(kdeh1);
        kdeh1_lp(tr,ti) = kdeh1.logprob(xv)/(ntest - 1);
    end
    
    save([figname '_results.mat']);
end

normflag = 1;

iid_m = mean(iid_lp,1);
iid_std = std(iid_lp,normflag,1);

ar_m = mean(ar_lp,1);
ar_std = std(ar_lp,normflag,1);

arh_m = mean(arh_lp,1);
arh_std = std(arh_lp,normflag,1);

h_m = mean(h_lp,1);
h_std = std(h_lp,normflag,1);

kde_m = mean(kde_lp,1);
kde_std = std(kde_lp,normflag,1);

kdem_m = mean(kdem_lp,1);
kdem_std = std(kdem_lp,normflag,1);

kdeh0_m = mean(kdeh0_lp,1);
kdeh0_std = std(kdeh0_lp,normflag,1);

kdeh1_m = mean(kdeh1_lp,1);
kdeh1_std = std(kdeh1_lp,normflag,1);

save([figname '_results.mat']);

% Plot results
dcolors = distinguishable_colors(8,{'w','k'});

figure(1);
clf;
hold on;

std_plot(ntrains,iid_m,iid_std,dcolors(1,:));
std_plot(ntrains,ar_m,ar_std,dcolors(2,:));
std_plot(ntrains,arh_m,arh_std,dcolors(3,:));
std_plot(ntrains,h_m,h_std,dcolors(4,:));
std_plot(ntrains,kde_m,kde_std,dcolors(5,:));
std_plot(ntrains,kdem_m,kdem_std,dcolors(6,:));
std_plot(ntrains,kdeh0_m,kdeh0_std,dcolors(7,:));
std_plot(ntrains,kdeh1_m,kdeh1_std,dcolors(8,:));

hold off;

set(gca,'XScale','log');

xlabel('Training sequence length $N_{\mathrm{train}}$');
ylabel('Held-out set per-sample log-probability');

legendstrs = {'Gaussian i.i.d.','AR','AR-HMM','HMM',...
    'KDE i.i.d.','KDE-MM','KDE-HMM $p{=}0$','KDE-HMM $p{=}1$'};
legend(legendstrs{:},'Location','NorthWest');

drawnow;

saveas(gcf,figname,'fig');
fig2ieeetran_fm(figname);

%function [hmm,hmm0,lps] = createHMM(ns,o,initvar,xs,inititer,minstep)
%
%[hmm,hmm0,lps] = MakeErgodicGaussHMM(initvar,xs,inititer,minstep);

%function [ar,ar0,lps] = createARHMM(ns,o,initvar,xs,inititer,minstep)
%
%[ar,ar0,lps] = MakeErgodicARHMM(initvar,o,xs,inititer,minstep);

%function [kdeh,kdeh0,lps] = createKDEHMM(ns,o,initvar,xs,inititer,minstep)
%
%[kdeh,kdeh0,lps] = MakeErgodicKDEHMM(initvar,o,xs,inititer,minstep);

function std_plot(ntrains,means,stds,col)

errorbar(ntrains,means,stds,'Color',col);

%semilogy(ntrains,means,'-',hmmstyle,'Color',col);
%
%ntps = numel(ntrains);
%for ti = 1:ntps,
%    semilogy(ntrains(ti)*[1 1],means(ti) + [-1 1]*stds(ti))
%end

function h = stationarize(h)

if (numel(h.OutputDistr) > 1),
    try
        h.StateGen = h.StateGen.setStationary;
    catch err
        warning('kdehmm_learning_experiment:stationarize',...
            ['Could not set stationary distribution due to error "'...
            err.message '"']);
    end
end