function [hmmt,hmm0,lpt,lpv,besthmm,besto,bestns,lpst,lpsr,lpdmin,...
    ttime,bestr] = train_block(hmmexample,createfun,trainempty,os,nss,...
    xs,initvars,inititer,miniter,minstep,xv)

ntrain = size(xs,2);
ntest = size(xv,2);

no = numel(os);
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt] = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr] = deal(cell(no,nnss));
[lpst{:}] = deal(-Inf);
[lpsr{:}] = deal(repmat({-Inf},1,nreinits));
[lpt,lpv] = deal(-Inf*ones(no,nnss));
lpdmin = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

parsize = [no,nnss];
maxparind = prod(parsize);
parfor parind = 1:maxparind,
    [o,ns] = ind2sub(parsize,parind);
    
    totaliter = 0;
    begintime = clock;
    begincpu = cputime;
    for r = 1:nreinits, % Train many initial models
        [hmmttmp,hmm0tmp,lpsr{parind}{r}]...
            = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
        
        totaliter = totaliter + numel(lpsr{parind}{r});
        dmintmp = nanmin(diff(lpsr{parind}{r}));
        if ~isempty(dmintmp),
            lpdmin(parind) = min(lpdmin(parind),dmintmp);
        end
        % Keep track of the most promising model
        if (lpsr{parind}{r}(end) > lpst{parind}(end)),
            bestr(parind) = r;
            hmmt(parind) = hmmttmp;
            hmm0(parind) = hmm0tmp;
            lpst{parind} = lpsr{parind}{r};
        end
    end
    
    % Perform additional training on the most promising candidate
    if trainempty,
        [hmmt(parind),lpstmp] = hmmt(parind).train([],[],miniter,minstep);
    else
        [hmmt(parind),lpstmp]...
            = hmmt(parind).train(xs,ntrain,miniter,minstep);
    end
    lpst{parind} = [lpst{parind} lpstmp];
    
    totaliter = totaliter + numel(lpstmp);
    dmintmp = nanmin(diff(lpst{parind}));
    if ~isempty(dmintmp),
        lpdmin(parind) = min(lpdmin(parind),dmintmp);
    end
    endtime = clock; % Record time required
    endcpu = cputime;
    
    % Reset initial distribution, since we assume stationarity
    if (ns > 1),% && ~any(any(isnan(hmmt(parind).StateGen.TransitionProb))),
        try
            hmmt(parind).StateGen = hmmt(parind).StateGen.setStationary;
        catch err
            warning('kdehmm_experiment:setStationary',...
                ['Could not set stationary distribution due to error "'...
                err.message '"']);
        end
    end
    
    % Evaluate performance of stationarized models
    lpt(parind) = hmmt(parind).logprob(xs)/(ntrain-os(o));
    lpv(parind) = hmmt(parind).logprob(xv)/(ntest-os(o));
    
    disp(['p=' int2str(os(o)) ', ns=' int2str(nss(ns))...
        ': lpt=' num2str(lpt(parind)) ', lpv=' num2str(lpv(parind))...
        ', secs=' num2str(etime(endtime,begintime))]);
    
    ttime(parind) = (endcpu - begincpu)/totaliter; % Store time per iteration
    
    % Plot progress
%%    clf;
%%    hold on;
%%    for r = 1:nreinits,
%%        plot(lpsr{parind}{r},'r');
%%    end
%%    plot(lpst{parind},'b');
%%    hold off;
%%    drawnow;
    
    %save([matfile '_train_block_' ints2str(parind)]);
end
